﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using FlowPreservation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Linq.Expressions;

namespace FlowDemoStore
{
    /// <summary>
    /// Simple test for causality chain preservation in Windows Store applications. 
    /// To see how causality is preserved, set breakpoints after awaits in asynchronous methods of this class.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// This event triggers a series of asynchronous operations over which causality is preserved.
        /// </summary>
        async protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            await ContextHelper.DropContext();
            await OuterAsync().WithCausality();
        }

        async private Task OuterAsync()
        {
            await Inner1Async().WithCausality();
            await Inner2Async().WithCausality();
            await Inner3Async().WithCausality();
        }

        async private Task Inner1Async()
        {
            await Task.Delay(100).WithCausality();
        }

        async private Task Inner2Async()
        {
            await Task.Yield().WithCausality();
        }

        async private Task Inner3Async()
        {
            await Inner3InnerAsync().WithCausality();
            await Task.Delay(100).WithCausality();
        }

        async private Task Inner3InnerAsync()
        {
            await Task.Delay(100).WithCausality();
            await Task.Yield().WithCausality();

            var semaphore = new SemaphoreSlim(0);
            semaphore.Wait();
        }
    }
}
