﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// Represents a loop in a causality chain
    /// </summary>
    struct StackTraceLoop
    {
        /// <summary>
        /// Loop constructor
        /// </summary>
        /// <param name="highIndex">Index of the most recent loop element</param>
        /// <param name="lowIndex">Index of the oldest loop element</param>
        /// <param name="count">Number of iterations of the loop</param>
        public StackTraceLoop(int highIndex, int lowIndex, int count)
        {
            if (highIndex < lowIndex)
                throw new AsyncFlowDiagnosticsException("highIndex is expected to be strictly higher than lowIndex: " + highIndex + ", " + lowIndex);

            HighIndex = highIndex;
            LowIndex = lowIndex;
            Count = count;
        }

        /// <summary>
        /// Index of the most recent loop element
        /// </summary>
        public readonly int HighIndex;

        /// <summary>
        /// Index of the oldest loop element
        /// </summary>
        public readonly int LowIndex;

        /// <summary>
        /// Number of iterations of the loop
        /// </summary>
        public readonly int Count;
    }
}
