﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// Stack trace segment
    /// </summary>
    class StackTraceSegment
    {
        // Pertaining to segments

        /// <summary>
        /// Frames belonging to this segment
        /// </summary>
        public readonly StackFrameSlim[] Frames;
        /// <summary>
        /// Hashes belonging to this segment
        /// </summary>
        public readonly long[] Hashes;
        
        // Pertaining to the whole sublist starting at this element

        /// <summary>
        /// Loops belonging to the whole list
        /// </summary>
        public readonly StackTraceLoop[] Loops;
        /// <summary>
        /// Total element count with loops collapsed in the entire list
        /// </summary>
        public readonly int TotalCount;
        /// <summary>
        /// Total element count in the entire list with loops expanded
        /// </summary>
        public readonly int TotalCountWithLoops;

        /// <summary>
        /// Constructor of a stack trace segment
        /// </summary>
        /// <param name="frames">Frames for the segment</param>
        /// <param name="hashesUpTo">Hashes for the segment</param>
        /// <param name="loops">Loops for the entire list</param>
        /// <param name="totalCount">Count with loops collapsed for the entire list</param>
        /// <param name="countWithLoops">Count with loops expanded for the entire list</param>
        public StackTraceSegment(StackFrameSlim[] frames, long[] hashesUpTo, StackTraceLoop[] loops, int totalCount, int countWithLoops)
        {
            this.Frames = frames;
            this.Hashes = hashesUpTo;
            this.Loops = loops;
            this.TotalCount = totalCount;
            this.TotalCountWithLoops = countWithLoops;
        }

        /// <summary>
        /// Frames count in this segment itself
        /// </summary>
        public int Count
        {
            get { return Frames.Length; }
        }
    }
}
