﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// Analog to MethodBase that stores its string representation in a ready-to-consume form for easy extraction
    /// </summary>
    struct MethodBaseSlim
    {
        internal MethodBaseSlim(string stringValue, bool isEventInfrastructure, bool isExternalCode)
        {
            this.StringValue = stringValue;
            this.IsEventInfrastructure = isEventInfrastructure;
            this.IsExternalCode = isExternalCode;
        }

        public MethodBaseSlim(MethodBase method)
        {
            this.StringValue = GetStringValue(method);
            this.IsEventInfrastructure = ExternalCodeHelper.IsEventInfrastructure(method);
            this.IsExternalCode = ExternalCodeHelper.IsExternalCode(method);
        }

        public readonly string StringValue;
        
        public readonly bool IsEventInfrastructure;
        public readonly bool IsExternalCode;

        /// <summary>
        /// Fancy ToString
        /// </summary>
        /// <param name="method">Method</param>
        /// <returns>String value</returns>
        private static string GetStringValue(MethodBase method)
        {
            if (method == null)
                return null;

            var builder = new StringBuilder();
            if (method.DeclaringType != null)
            {
                builder.Append(method.DeclaringType.FullName);
                builder.Append(".");
            }
            builder.Append(method.Name);
            if (method.IsGenericMethod)
            {
                builder.Append("[");
                var genericArguments = method.GetGenericArguments();
                for (int i = 0; i < genericArguments.Length; i++)
                {
                    if (i > 0)
                        builder.Append(",");
                    builder.Append(genericArguments[i]);
                }
                builder.Append("]");
            }
            builder.Append("(");
            var parameters = method.GetParameters();
            for (int i = 0; i < parameters.Length; i++)
            {
                if (i > 0)
                    builder.Append(", ");
                builder.Append((parameters[i] != null ? parameters[i].ParameterType.Name : "<UnknownType>")
                + (string.IsNullOrEmpty(parameters[i].Name) ? string.Empty : " " + parameters[i].Name));
            }
            builder.Append(")");
            return builder.ToString();
        }
    }
}
