﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// Wraps a void-returning ICriticalNotifyCompletion-based awaitable to execute an action 
    /// after asynchronous part is completed, but before it schedules the continuation
    /// </summary>
    /// <typeparam name="TAwaiter">Awaiter type</typeparam>
    public struct CriticalAppendableAwaitable<TAwaiter> where TAwaiter : ICriticalNotifyCompletion
    {
        private readonly Func<TAwaiter> getAwaiter;
        private readonly Action preCompleted;
        private readonly Func<TAwaiter, bool> isCompleted;
        private readonly Action<TAwaiter> getResult;

        public CriticalAppendableAwaitable(Func<TAwaiter> getAwaiter, Action preCompleted, 
            Func<TAwaiter, bool> isCompleted, Action<TAwaiter> getResult)
        {
            this.getAwaiter = getAwaiter;
            this.preCompleted = preCompleted;
            this.isCompleted = isCompleted;
            this.getResult = getResult;
        }

        public CriticalAppendableAwaiter<TAwaiter> GetAwaiter()
        {
            return new CriticalAppendableAwaiter<TAwaiter>(getAwaiter(), preCompleted, isCompleted, getResult);
        }
    }

    /// <summary>
    /// Wraps a result-returning ICriticalNotifyCompletion-based awaitable to execute an action 
    /// after asynchronous part is completed, but before it schedules the continuation
    /// </summary>
    /// <typeparam name="TAwaiter">Awaiter type</typeparam>
    /// <typeparam name="TResult">Result type</typeparam>
    public struct CriticalAppendableAwaitable<TAwaiter, TResult> where TAwaiter : ICriticalNotifyCompletion
    {
        private readonly Func<TAwaiter> getAwaiter;
        private readonly Action preCompleted;
        private readonly Func<TAwaiter, bool> isCompleted;
        private readonly Func<TAwaiter, TResult> getResult;

        public CriticalAppendableAwaitable(Func<TAwaiter> getAwaiter, Action preCompleted,
            Func<TAwaiter, bool> isCompleted, Func<TAwaiter, TResult> getResult)
        {
            this.getAwaiter = getAwaiter;
            this.preCompleted = preCompleted;
            this.isCompleted = isCompleted;
            this.getResult = getResult;
        }

        public CriticalAppendableAwaiter<TAwaiter, TResult> GetAwaiter()
        {
            return new CriticalAppendableAwaiter<TAwaiter, TResult>(getAwaiter(), preCompleted, isCompleted, getResult);
        }
    }
}
