﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// Immediately schedules continuation onto default synchronization context
    /// </summary>
    public static class ContextHelper
    {
        public static DropContextAwaitable DropContext()
        {
            return default(DropContextAwaitable);
        }
    }

    /// <summary>
    /// Analog to YieldAwaitable that does not preserve context
    /// </summary>
    public struct DropContextAwaitable
    {
        public DropContextAwaiter GetAwaiter()
        {
            return default(DropContextAwaiter);
        }
    }

    /// <summary>
    /// Immediately schedules continuation onto default synchronization context
    /// </summary>
    public struct DropContextAwaiter : INotifyCompletion
    {
        private static readonly SynchronizationContext DefaultContext = new SynchronizationContext();

        public bool IsCompleted { get { return false; } }

        public void GetResult() { }

        public void OnCompleted(Action continuation)
        {
            DefaultContext.Post(_ => continuation(), null);
        }
    }
}
