﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics.Tracing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservation
{
    /// <summary>
    /// A set of constants from TplEtwProvider and FrameworkEventSource
    /// </summary>
    public static class EventConstants
    {
        /// <summary>
        /// An excerpt from TplEtwProvider
        /// </summary>
        public abstract class Tpl
        {
            public static readonly Guid GUID = new Guid("2e5dba47-a3d2-4d16-8ee0-6671ffdcd7b5");

            public const int PARALLELLOOPBEGIN_ID = 1;
            public const int PARALLELLOOPEND_ID = 2;

            public const int PARALLELINVOKEBEGIN_ID = 3;
            public const int PARALLELINVOKEEND_ID = 4;

            public const int PARALLELFORK_ID = 5;
            public const int PARALLELJOIN_ID = 6;

            public const int TASKSCHEDULED_ID = 7;

            public const int TASKSTARTED_ID = 8;
            public const int TASKCOMPLETED_ID = 9;

            public const int TASKWAITBEGIN_ID = 10;
            public const int TASKWAITEND_ID = 11;

            // Tasks.Loop == 1
            public abstract void ParallelLoopBegin(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID, ForkJoinOperationType OperationType, long InclusiveFrom, long ExclusiveTo);
            public abstract void ParallelLoopEnd(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID, long TotalIterations);

            // Tasks.Invoke == 2
            public abstract void ParallelInvokeBegin(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID, ForkJoinOperationType OperationType, int ActionCount);
            public abstract void ParallelInvokeEnd(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID);

            // Tasks.ForkJoin == 5
            public abstract void ParallelFork(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID);
            public abstract void ParallelJoin(int OriginatingTaskSchedulerID, int OriginatingTaskID, int ForkJoinContextID);

            // No task
            public abstract void TaskScheduled(int OriginatingTaskSchedulerID, int OriginatingTaskID, int TaskID, int CreatingTaskID, int TaskCreationOptions);

            // Tasks.TaskExecute == 3
            public abstract void TaskStarted(int OriginatingTaskSchedulerID, int OriginatingTaskID, int TaskID);
            public abstract void TaskCompleted(int OriginatingTaskSchedulerID, int OriginatingTaskID, int TaskID, bool IsExceptional);

            // Tasks.TaskWait == 4
            public abstract void TaskWaitBegin(int OriginatingTaskSchedulerID, int OriginatingTaskID, int TaskID, TaskWaitBehavior Behavior);
            public abstract void TaskWaitEnd(int OriginatingTaskSchedulerID, int OriginatingTaskID, int TaskID);

            public enum ForkJoinOperationType
            {
                ParallelInvoke = 1,
                ParallelFor = 2,
                ParallelForEach = 3,
            }

            public class Tasks
            {
                public const EventTask Loop = ((EventTask)1);
                public const EventTask Invoke = ((EventTask)2);
                public const EventTask ForkJoin = ((EventTask)5);
                public const EventTask TaskExecute = ((EventTask)3);
                public const EventTask TaskWait = ((EventTask)4);
            }

            public enum TaskWaitBehavior
            {
                Synchronous = 1,
                Asynchronous = 2,
            }
        }

        /// <summary>
        /// An excerpt frm FrameworkEventSource
        /// </summary>
        public abstract class Framework
        {
            public static readonly Guid GUID = new Guid("8E9F5090-2D75-4d03-8A81-E5AFBF85DAF1");

            public const int THREADPOOLDEQUEUEWORK_ID = 31;
            public const int THREADPOOLENQUEUEWORK_ID = 30;

            // Keywords.ThreadPool == 2L
            public abstract void ThreadPoolDequeueWork(long workID);
            public abstract void ThreadPoolEnqueueWork(long workID);

            public static class Keywords
            {
                public const EventKeywords Loader = (EventKeywords)1L;
                public const EventKeywords ThreadPool = (EventKeywords)2L;
            }
        }
    }
}
