﻿using FlowPreservation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FlowPreservationTests
{
    [TestClass]
    public class AsyncLocalTests
    {
        [TestMethod]
        async public Task AsyncLocalBasic()
        {
            await VerifyValueFlow(InnerAsync);
        }

        [TestMethod]
        async public Task AsyncLocalNested()
        {
            await VerifyValueFlow(OuterAsync);
        }

        async private Task VerifyValueFlow(Func<Action, Task> action)
        {
            await ContextHelper.DropContext();
            const string expectedValue = "Hello!";
            var asyncLocal = new AsyncLocal<string>();
            asyncLocal.Value = expectedValue;
            int initialThreadId = Thread.CurrentThread.ManagedThreadId;
            do await action(() => Assert.AreEqual(expectedValue, asyncLocal.Value));
            while (Thread.CurrentThread.ManagedThreadId == initialThreadId);
            VerifyNoLeftovers(initialThreadId, asyncLocal);
        }

        private void VerifyNoLeftovers<T>(int initialThreadId, AsyncLocal<T> asyncLocal)
        {
            var countdown = new CountdownEvent(10);
            var releaser = new ManualResetEventSlim(false);
            bool verifiedAny = false;
            var tasks = new Task[countdown.InitialCount];
            for (int i = 0; i < tasks.Length; i++)
                tasks[i] = Task.Run(() =>
                {
                    if (Thread.CurrentThread.ManagedThreadId == initialThreadId)
                    {
                        Assert.AreEqual(default(T), asyncLocal.Value, "Leftover value stored on old thread");
                        Volatile.Write(ref verifiedAny, true);
                    }
                    countdown.Signal();
                    releaser.Wait();
                });
            countdown.Wait();
            releaser.Set();
            Assert.IsTrue(verifiedAny, "Could not verify absence of leftover value: no task was scheduled to initial thread");
        }

        async private Task InnerAsync(Action verify)
        {
            await Task.Delay(1);
            verify();
        }

        async private Task OuterAsync(Action verify)
        {
            await InnerAsync(verify);
            verify();
        }
    }
}
