﻿using FlowPreservation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservationTests
{
    /// <summary>
    /// Assertions regarding frame presence on stack
    /// </summary>
    internal static class AssertEx
    {
        private static readonly string[] topMethods = 
        {
            "Microsoft.VisualStudio.TestTools.TestTypes.Unit.UnitTestExecuter.RunTestMethod",
            "Microsoft.VisualStudio.TestPlatform.MSTestFramework.UnitTestRunner.RunSingleTest",
        };
        private static readonly char[] newLineChars = { '\r', '\n' };

        private static string FormatTimes(string verb, int count)
        {
            return string.Format("was{0}{1}{2}", count == 0 ? " not" : "", " " + verb, count == 1 ? " once" : count > 1 ? " " + count + " times" : "");
        }

        private static string FormatPresence(string str, int expectedCount, int actualCount)
        {
            return string.Format("\"{0}\" {1}, but {2}",
                str, FormatTimes("expected", expectedCount), FormatTimes("present", actualCount));
        }

        private static int GetOnStackCount(string str)
        {
            string stack = FlowReservoir.ExtendedStack;
            var lines = stack.Split(newLineChars, StringSplitOptions.RemoveEmptyEntries);
            int actualCount = lines.Count(l => l.Contains(str));
            return actualCount;
        }

        /// <summary>
        /// String is on stack for exactly expected number of times
        /// </summary>
        /// <param name="str">String</param>
        /// <param name="expectedCount">How many times</param>
        public static void OnStackCount(string str, int expectedCount)
        {
            int actualCount = GetOnStackCount(str);
            Assert.IsTrue(expectedCount == actualCount, FormatPresence(str, expectedCount, actualCount) 
                + " on stack:" + Environment.NewLine + FlowReservoir.ExtendedStack);
        }

        /// <summary>
        /// Method is not on stack (as a state machine)
        /// </summary>
        /// <param name="method">Method</param>
        public static void NotOnStack(Func<Task> method)
        {
            OnStackCount("<" + method.Method.Name + ">", 0);
        }

        /// <summary>
        /// A unit test runner method is present on the bottom of the stack (that means that the chain did not tear down)
        /// </summary>
        public static void StackHasRoot()
        {
            var actualCounts = new int[topMethods.Length];
            for (int i = 0; i < topMethods.Length; i++)
            {
                actualCounts[i] = GetOnStackCount(topMethods[i]);
                if (actualCounts[i] == 1)
                    return;
            }
            Assert.Fail(string.Join("," + Environment.NewLine,
                topMethods.Zip(actualCounts, (t, a) => FormatPresence(t, 1, a)))
                + Environment.NewLine + "on stack:" + Environment.NewLine + FlowReservoir.ExtendedStack);
        }
    }
}
