﻿using FlowPreservation;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FlowPreservationTests
{
    /// <summary>
    /// Asynchronous methods called by tests
    /// </summary>
    static class Methods
    {
        async public static Task FirstAsync()
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
        }

        async public static Task SecondAsync()
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
        }

        async public static Task CommonAsync()
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
        }

        async public static Task InfiniteAsync()
        {
            await Task.Delay(int.MaxValue);
            AssertEx.StackHasRoot();
        }

        async public static Task NestedOuter()
        {
            await NestedInner();
            AssertEx.StackHasRoot();
        }

        async public static Task NestedInner()
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
        }

        async public static Task Loop()
        {
            const int iterations = 6;
            for (int i = 0; i < iterations; i++)
            {
                await FirstAsync();
                await SecondAsync();
            }
            AssertEx.StackHasRoot();
            AssertEx.OnStackCount((iterations - 1) + " times", 1);
        }


        async public static Task Flaky(int iterations)
        {
            for (int i = 0; i < iterations; i++)
            {
                await Task.Delay(TimeSpan.FromTicks(1));
            }
            AssertEx.StackHasRoot();
        }

        async public static Task<int> FirstBlockAction(int arg)
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
            return arg + 1;
        }

        async public static Task<int> SecondBlockAction(int arg)
        {
            await Task.Delay(1);
            AssertEx.StackHasRoot();
            return arg + 1;
        }

        async public static Task FirstSimple()
        {
            await Task.Delay(1);
        }

        async public static Task SecondSimple()
        {
            await Task.Delay(1);
        }
    }
}
