﻿using EnvDTE;
using EnvDTE80;
using FlowPreservation;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Reflection;

namespace FlowViewer
{
    /// <summary>
    /// Add-in that outputs causality chain for current thread into output window. 
    /// </summary>
    class FlowDumper : PluginBase
    {
        public FlowDumper(DTE2 dte, AddIn addIn) : base(dte, addIn, true) { }

        /// <summary>
        /// Displays stored causality chain for current thread in the Output window
        /// </summary>
        protected override void Run()
        {
            var window = dte.ToolWindows.OutputWindow;
            window.Parent.Activate();

            const string asyncFlowPaneName = "AsyncFlow";
            var pane = window.OutputWindowPanes.Cast<OutputWindowPane>().Any(p => p.Name == asyncFlowPaneName)
                ? window.OutputWindowPanes.Item(asyncFlowPaneName)
                : window.OutputWindowPanes.Add(asyncFlowPaneName);
            pane.Activate();

            try
            {
                var chain = GetFullCausalityChain(dte.Debugger.CurrentThread);
                pane.OutputString("Causality chain for thread " + dte.Debugger.CurrentThread.ID + ":" + eol + eol);
                pane.OutputString(chain + eol + eol);
            }
            catch (Exception ex)
            {
                pane.OutputString(ex + eol + eol);
            }
        }
    }
}
 