﻿using EnvDTE;
using EnvDTE80;
using FlowPreservation;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FlowViewer
{
    /// <summary>
    /// DGML diagram generator, which presents causality data for all threads in a Parallel Stacks-like view
    /// </summary>
    class FlowGrapher : PluginBase
    {
        public FlowGrapher(DTE2 dte, AddIn addIn) : base(dte, addIn, true) { }

        /// <summary>
        /// Generates DGML diagram with all user-code frames of all threads interconnected by causality matters
        /// and opens it in the IDE.
        /// </summary>
        protected override void Run()
        {
            try
            {
                var graph = new StackGraph();
                foreach (Thread thread in dte.Debugger.CurrentProgram.Threads)
                {
                    var chain = GetFullCausalityChain(thread);
                    if (!string.IsNullOrWhiteSpace(chain))
                        graph.AddSegment(chain);
                }
                graph.Collapse();

                var dgml = graph.GetDgml();
                var fileName = Path.GetTempFileName() + ".dgml";
                dgml.Save(fileName);
                dte.ItemOperations.OpenFile(fileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }
    }
}
 