﻿using EnvDTE;
using EnvDTE80;
using FlowPreservation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FlowViewer
{
    /// <summary>
    /// Extracts stored causality information for Windows Store applications.
    /// </summary>
    class ModernStackHelper : StackHelperBase
    {
        public ModernStackHelper(DTE2 dte) : base(dte) { }

        /// <summary>
        /// Extracts causality chain for the current thread by evaluating field expressions in the debugger
        /// </summary>
        /// <returns>Stored causality chain except for the most recent stack segment, which is still on the stack</returns>
        public override string GetCausalityChain()
        {
            var builder = new StringBuilder();
            var frameStorage = dte.Debugger.GetExpression(typeof(FrameStorage).FullName);
            var frames = frameStorage.DataMember(typeof(FrameStorage), typeof(AsyncLocal<LinkedList<string>>));
            var framesGuid = CreateGuid(frames.DataMember(typeof(AsyncLocal<LinkedList<string>>), typeof(Guid)));
            var discardableStorage = frames.DataMember(string.Format("base {{{0}}}", typeof(AsyncLocal).FullName))
                .WithStaticMembers().DataMember("DiscardableThreadStorage");
            var entries = discardableStorage.WithNonPublicMembers().DataMember("entries");
            var entry = entries.DataMember(e => CreateGuid(e.DataMember("key")) == framesGuid);
            var entryValue = entry.DataMember("value");
            int count = int.Parse(entryValue.WithNonPublicMembers().DataMember("count").Value);
            var currentNode = entryValue.WithNonPublicMembers().DataMember("head");
            for (int i = 0; i < count; i++)
            {
                var frame = StringParse(currentNode.WithNonPublicMembers().DataMember("item"));
                if (i != count - 1 || !string.IsNullOrWhiteSpace(frame))
                    builder.AppendLine(frame);
                currentNode = currentNode.WithNonPublicMembers().DataMember("next");
            }
            return builder.ToString();
        }

        private Guid CreateGuid(Expression expression)
        {
            var nonPublic = expression.WithNonPublicMembers();
            return new Guid(
                int.Parse(nonPublic.DataMember("_a").Value),
                short.Parse(nonPublic.DataMember("_b").Value),
                short.Parse(nonPublic.DataMember("_c").Value),
                byte.Parse(nonPublic.DataMember("_d").Value),
                byte.Parse(nonPublic.DataMember("_e").Value),
                byte.Parse(nonPublic.DataMember("_f").Value),
                byte.Parse(nonPublic.DataMember("_g").Value),
                byte.Parse(nonPublic.DataMember("_h").Value),
                byte.Parse(nonPublic.DataMember("_i").Value),
                byte.Parse(nonPublic.DataMember("_j").Value),
                byte.Parse(nonPublic.DataMember("_k").Value));
        }
    }
}
