﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;

namespace FlowViewer
{
    /// <summary>
    /// Helps extract member names from expressions
    /// </summary>
    static class ReflectionHelper
    {
        /// <summary>
        /// Finds the name of the field or property
        /// </summary>
        /// <typeparam name="T">Declaring type</typeparam>
        /// <typeparam name="M">Member type</typeparam>
        /// <param name="getter">Linq expression representing member access</param>
        /// <returns>Field or property name</returns>
        public static string MemberName<T, M>(Expression<Func<T, M>> getter)
        {
            var memberExpression = getter.Body as MemberExpression;
            if (memberExpression == null)
                throw new ArgumentException("expression is not MemberExpression", "expression");

            return memberExpression.Member.Name;
        }

        /// <summary>
        /// Finds field name by field type
        /// </summary>
        /// <param name="declaringType">Declaring type</param>
        /// <param name="fieldType">Field type (there must be exactly one field of this type defined on declaring type)</param>
        /// <returns>Field name</returns>
        public static string FieldName(Type declaringType, Type fieldType)
        {
            return declaringType.GetTypeInfo().DeclaredFields.Single(f => f.FieldType == fieldType).Name;
        }
    }
}
